<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 */

namespace Search_Filter;

use Search_Filter\Core\CSS_Loader;
use Search_Filter\Core\Icons;
use Search_Filter\Core\Scripts;
use Search_Filter\Core\SVG_Loader;
use Search_Filter\Util;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * The main class for initialising all things for the frontend.
 */
class Frontend {

	/**
	 * The ID of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    3.0.0
	 * @param string $plugin_name  The name of the plugin.
	 * @param string $version      The version of this plugin.
	 */

	private $registered_styles  = array();
	private $registered_scripts = array();

	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;

	}
	public function init() {
		\Search_Filter\Query\Selector::init();
		\Search_Filter\Query::init();
	}
	public function register_styles() {

		Icons::register();

		$this->registered_styles = array(
			$this->plugin_name                => array(
				'src'     => Scripts::get_frontend_assets_url() . 'css/frontend/frontend.' . Util::get_file_ext( 'css' ),
				'deps'    => array( $this->plugin_name . '-flatpickr' ),
				'version' => $this->version,
				'media'   => 'all',
			),
			// TODO - use the HMR url? Scripts::get_frontend_assets_url()
			$this->plugin_name . '-flatpickr' => array(
				'src'     => trailingslashit( plugin_dir_url( dirname( __FILE__ ) ) ) . 'assets/css/vendor/flatpickr.' . Util::get_file_ext( 'css' ),
				'deps'    => array(),
				'version' => $this->version,
				'media'   => 'all',
			),
		);

		if ( CSS_Loader::get_mode() === 'file-system' ) {
			$this->registered_styles[ $this->plugin_name . '-ugc-styles' ] = array(
				'src'     => CSS_Loader::get_css_url(),
				'deps'    => array( $this->plugin_name ),
				'version' => CSS_Loader::get_version(),
				'media'   => 'all',
			);
		}

		$this->registered_styles = apply_filters( 'search-filter/frontend/register_styles', $this->registered_styles );

		foreach ( $this->registered_styles as $handle => $args ) {
			wp_register_style( $handle, $args['src'], $args['deps'], $args['version'], $args['media'] );
		}
	}
	public function register_scripts() {

		$this->registered_scripts = array(
			$this->plugin_name        => array(
				'src'     => Scripts::get_frontend_assets_url() . 'js/frontend/frontend.' . Util::get_file_ext( 'js' ),
				'deps'    => array( 'search-filter-flatpickr' ),
				'version' => $this->version,
				'footer'  => false,
			),
			// TODO - use the HMR url? Scripts::get_frontend_assets_url()
			'search-filter-flatpickr' => array(
				'src'     => trailingslashit( plugin_dir_url( dirname( __FILE__ ) ) ) . 'assets/js/vendor/flatpickr.' . Util::get_file_ext( 'js' ),
				'deps'    => array(),
				'version' => $this->version,
				'footer'  => false,
			),
		);

		$this->registered_scripts = apply_filters( 'search-filter/frontend/register_scripts', $this->registered_scripts );

		foreach ( $this->registered_scripts as $handle => $args ) {
			wp_register_script( $handle, $args['src'], $args['deps'], $args['version'], $args['footer'] );
		}
	}
	/**
	 * Register the stylesheets for the public-facing side of the plugin.
	 *
	 * @since    3.0.0
	 */
	public function enqueue_styles() {
		$this->register_styles();

		// TODO - need to re-implement dynamic file extension - we're not building unminifed files.

		$enqueued_styles = array_keys( $this->registered_styles );

		$enqueued_styles = apply_filters( 'search-filter/frontend/enqueue_styles', $enqueued_styles );

		foreach ( $enqueued_styles as $handle ) {
			if ( wp_style_is( $handle, 'registered' ) ) {
				wp_enqueue_style( $handle );
			}
		}
	}
	/**
	 * Register the JavaScript for the frontend
	 *
	 * @since    3.0.0
	 */
	public function enqueue_scripts() {

		$this->register_scripts();

		$enqueued_scripts = array_keys( $this->registered_scripts );
		$enqueued_scripts = apply_filters( 'search-filter/frontend/enqueue_scripts', $enqueued_scripts );

		foreach ( $enqueued_scripts as $handle ) {
			if ( wp_script_is( $handle, 'registered' ) ) {
				wp_enqueue_script( $handle );
			}
		}
		Scripts::attach_globals(
			$this->plugin_name,
			'frontend',
			array(
				'fields'     => (object) array(),
				'queries'    => (object) array(),
				'components' => (object) array(),
				'modules'    => (object) array(),
			)
		);

		/*
		 * Figure out if we need to enqueue the scripts now or later
		 */

		// TODO - re-implement lazy loading of assets.
		// if the user has set lazy load JS, then we won't want to load here
		// $lazy_load_js = Util::get_option( 'search_filter_lazy_load_js' );.

		// checks whether the user has enabled  CSS & JS (they can disabled in settings)
		// $load_js_css = Util::get_option( 'search_filter_load_js_css' );.

		// If lazy load js is off, and load css & js files is on.

		/*
		if( ( 1 != $lazy_load_js ) && ( 1 == $load_js_css ) ) {
			$this->enqueue_scripts();
		}
		*/
	}
	/**
	 * Outputs the JSON object for the active fields (inital render data), queries and template data.
	 */
	public function data() {
		$data = array(
			'fields'  => Fields::get_active_fields(),
			'queries' => Fields::get_active_queries(),
		);
		// Add filter to modify:
		$data = apply_filters( 'search-filter/frontend/data', $data );

		?>
		<script type="text/javascript">
			window.searchAndFilterData = <?php echo wp_json_encode( $data ); ?>;
		</script>
		<?php
	}

}
