<?php
/**
 * Util class
 *
 * @link       https://searchandfilter.com
 * @since      3.0.0
 * @package    Search_Filter
 */

namespace Search_Filter;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * A helper class with functions used across the plugin
 */
class Util {
	/**
	 * Stores a copy of any options retrieved to save additional function calls to the same options later
	 * in page processing
	 *
	 * @var array
	 */
	private static $options = array();

	/**
	 * Stores a copy of the queries to save additional function calls for the same info later
	 *
	 * @var array
	 */
	private static $queries = array();

	/**
	 * Adds `.min` to a file extension if SCRIPT_DEBUG is disabled
	 *
	 * @param string $file_ext  The extension of the file.
	 *
	 * @return string
	 */
	public static function get_file_ext( $file_ext ) {

		// TODO - need to re-implement dynamic file extension.

		$file_ext = strtolower( $file_ext );

		// TODO.
		/*
		 if ( false === SCRIPT_DEBUG ) {
			$file_ext = '.min' . $file_ext;
		} */

		return $file_ext;
	}


	/**
	 * Get the data for the object that gets passed to JS app
	 *
	 * @return array
	 */
	public static function get_js_data() {
		return array(
			'ajaxUrl'      => admin_url( 'admin-ajax.php' ),
			'restUrl'      => rest_url( 'search-filter' ),
			'homeUrl'      => home_url( '/' ),
			'dashboardUrl' => admin_url( 'admin.php?page=search-filter' ),
		);
	}


	/**
	 * Wrapper for the WP `get_option` function, implementing defaults if they do not exist yet
	 *
	 * @param string $option_name  The option key required.
	 *
	 * @return mixed  The value for the option
	 */
	public static function get_option( $option_name ) {

		// check to see if we've looked this up before, if so return the existing value.
		if ( isset( self::$options[ $option_name ] ) ) {
			return self::$options[ $option_name ];
		}

		// todo - set defaults externally.
		$option_defaults = array(
			'search_filter_lazy_load_js' => 0,
			'search_filter_load_js_css'  => 1,
		);

		$option_value = get_option( $option_name );

		// if option is not set, and there is a default for it, use the default.
		if ( ( false === $option_value ) && ( isset( $option_defaults[ $option_name ] ) ) ) {
			$option_value = $option_defaults[ $option_name ];
		}

		self::$options[ $option_name ] = $option_value;

		return $option_value;
	}

	/**
	 * Converts an associative array to a HTML attribute string
	 *
	 * @param array $attributes  An associative array of key -> value pairs.
	 *
	 * @return string
	 */
	public static function get_attributes_html( $attributes ) {

		$html = '';

		if ( is_array( $attributes ) ) {
			foreach ( $attributes as $attribute_name => $value ) {

				$clean_value = '';
				if ( is_array( $value ) ) {
					$clean_value = esc_attr( wp_json_encode( $value ) );
				} else {
					$clean_value = esc_attr( $value );
				}

				// Make sure the attibute + value are not empty.
				if ( ( ! empty( $attribute_name ) ) && ( $clean_value !== '' ) ) {
					$html .= ' ' . $attribute_name . '="' . $clean_value . '" ';
				} elseif ( ! empty( $attribute_name ) ) {
					$html .= ' ' . $attribute_name . ' ';
				}
			}
		}

		return $html;
	}

	/**
	 * Insert an assoc array into another assoc array by array key.
	 *
	 * @param array $source  Source array.
	 * @param array $additions Array of key -> value pairs to add to the source array.
	 * @param array $target Target position.
	 * @return array The new array with the additions.
	 */
	public static function array_insert_assoc( $source, $additions, $target ) {

		$defaults = array(
			'key'      => '',
			'position' => 'after',
		);

		$args = wp_parse_args( $target, $defaults );

		// Rebuild the array in order, inserting our new options before/after the desired target.
		$new_array = array();
		foreach ( $source as $key => $option ) {

			if ( 'before' === $args['position'] ) {
				if ( $args['key'] === $key ) {
					// We found the key, merge the additions before we add it.
					$new_array = array_merge( $new_array, $additions );
				}
			}

			// Copy the option and key.
			$new_array[ $key ] = $option;

			if ( 'after' === $args['position'] ) {
				if ( $args['key'] === $key ) {
					// We found the key, merge the additions.
					$new_array = array_merge( $new_array, $additions );
				}
			}
		}
		return $new_array;
	}

	// A box attribute is one that is a string of 4 values, separated by spaces.
	// $value will be an array of 4 values, with keys "top", "right", "bottom", "left".
	public static function sanitize_css_box( $value ) {

		if ( ! is_array( $value ) ) {
			return '';
		}

		// Make sure we have all 4 values.
		if ( 4 !== count( $value ) ) {
			return '';
		}

		$clean_value = '';
		$key_order   = array( 'top', 'right', 'bottom', 'left' );
		foreach ( $key_order as $key ) {
			if ( ! isset( $value[ $key ] ) ) {
				return '';
			}
			// Sanitize the value (can be a valid CSS unit).
			$value[ $key ] = sanitize_title_with_dashes( $value[ $key ] );
			$clean_value  .= $value[ $key ] . ' ';
		}

		return trim( $clean_value );
	}
}
