<?php
/**
 * Scripts Class
 *
 * @link       https://searchandfilter.com
 * @since      3.0.0
 * @package    Search_Filter
 * @subpackage Search_Filter/Core
 */

namespace Search_Filter\Core;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * A Scripts utility class to be used by both frontend and admin.
 */
class Scripts {

	/**
	 * Track whether we've already declared the global `window.searchAndFilter` object.
	 */
	private static $has_init_global = false;

	/**
	 * If the global has not been init yet, then init it.
	 */
	public static function init_global_js() {
		ob_start();
		if ( ! self::$has_init_global ) {
			self::$has_init_global = true;
			?>
			window.searchAndFilter = {};
			<?php
		}
		return trim( ob_get_clean() );
	}
	/**
	 * If the global has not been init yet, then init it.
	 */
	public static function init_view_js() {
		ob_start();
		if ( ! self::$has_init_global ) {
			self::$has_init_global = true;
			?>
			window.searchAndFilter = {};
			<?php
		}
		return trim( ob_get_clean() );
	}

	/**
	 * Attach a global variables to the window object and add them inline to a script handle.
	 *
	 * They will always be attached to the `window.searchAndFilter` object then under the name of the view.
	 *
	 * @param string $script_handle The name of the script to attach the global to.
	 * @param string $view          The name of the global variable.
	 * @param array  $data          The data to attach to the global.
	 * @param string $position      The position of the inline script in relation the the handle.
	 */
	public static function attach_globals( $script_handle, $view, $data, $position = 'before' ) {
		$output_js[ $view ] = $data;
		ob_start();
		?>
		window.searchAndFilter.<?php echo sanitize_key( $view ); ?> = <?php echo wp_json_encode( $data ); ?>;
		<?php
		$js = self::init_global_js() . trim( ob_get_clean() );
		wp_add_inline_script( $script_handle, $js, $position );
	}
	/**
	 * Gets the plugins assets URL path.
	 *
	 * @return string The URL path to the assets.
	 */
	public static function get_admin_assets_url() {
		// TODO - use wp function for getting the url (rather than SEARCH_FILTER_URL)
		// eg - plugin_dir_url( dirname( __FILE__ ) ) . 'assets/css/vendor/flatpickr.css' ).
		$assets_url = SEARCH_FILTER_URL . 'assets/';
		if ( defined( 'SEARCH_FILTER_ADMIN_ASSETS_URL' ) ) {
			$assets_url = SEARCH_FILTER_ADMIN_ASSETS_URL;
		}
		return $assets_url;
	}
	/**
	 * Gets the plugins frontend assets URL path.
	 *
	 * @return string The URL path to the assets.
	 */
	public static function get_frontend_assets_url() {
		$assets_url = SEARCH_FILTER_URL . 'assets/';
		if ( defined( 'SEARCH_FILTER_FRONTEND_ASSETS_URL' ) ) {
			$assets_url = SEARCH_FILTER_FRONTEND_ASSETS_URL;
		}
		return $assets_url;
	}
	/**
	 * Preload the API requests.
	 *
	 * @param array $preload_paths The paths to preload.
	 */
	public static function preload_api_requests( $preload_paths ) {
		/* Copied from core - wp-includes/block-editor.php */

		// Restore the global $post as it was before API preloading.
		// Preload common data.
		global $post, $wp_scripts, $wp_styles, $post_id;

		/*
		* Ensure the global $post, $wp_scripts, and $wp_styles remain the same after
		* API data is preloaded.
		* Because API preloading can call the_content and other filters, plugins
		* can unexpectedly modify the global $post or enqueue assets which are not
		* intended for the block editor.
		*/
		$backup_global_post = ! empty( $post ) ? clone $post : $post;
		$backup_wp_scripts  = ! empty( $wp_scripts ) ? clone $wp_scripts : $wp_scripts;
		$backup_wp_styles   = ! empty( $wp_styles ) ? clone $wp_styles : $wp_styles;
		$backup_post_id     = $post_id;

		$preload_data = array_reduce(
			$preload_paths,
			'rest_preload_api_request',
			array()
		);
		//phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
		$post = $backup_global_post;
		//phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
		$wp_scripts = $backup_wp_scripts;
		//phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
		$wp_styles = $backup_wp_styles;
		//phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
		$post_id = $backup_post_id;

		wp_add_inline_script(
			'wp-api-fetch',
			sprintf(
				'wp.apiFetch.use( wp.apiFetch.createPreloadingMiddleware( %s ) );',
				wp_json_encode( $preload_data )
			),
			'after'
		);
		wp_add_inline_script(
			'wp-api-fetch',
			'window.apiResult =  ' . wp_json_encode( $preload_data ),
			'after'
		);
	}
}
