<?php
/**
 * Class for handling the frontend display of a field.
 *
 * @link       http://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 * @subpackage Search_Filter/public
 */

namespace Search_Filter\Fields;

use Search_Filter\Core\CSS_Loader;
use Search_Filter\Util;
use Search_Filter\Database\Queries\Fields as Field_Query;
use Search_Filter\Fields;
use Search_Filter\Styles;
use Search_Filter\Record_Base;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

// TODO - do we need to split this into a model / view?

/**
 * The base field for frontend fields (should be extended).
 */
class Field extends Record_Base {
	/**
	 * The record store name
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      int    $id    ID
	 */
	public $record_store = 'field';

	/**
	 * The meta type for the meta table.
	 *
	 * @var string
	 */
	public static $meta_table = 'search_filter_field';
	/**
	 * The full string of the class name of the query class for this section.
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $records_class    The string class name.
	 */
	public static $records_class = 'Search_Filter\\Database\\Queries\\Fields';

	/**
	 * The class name to handle interacting with the record stores.
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $base_class    ID
	 */
	public static $base_class = 'Search_Filter\\Fields';
	/**
	 * Reference to the query ID
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      integer    $query_id    Maintains and registers all hooks for the plugin.
	 */
	private $query_id = 0;

	/**
	 * Unique ID for referencing on the frontend
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      int    $id    ID
	 */
	protected $uid = 0;

	/**
	 * Array of icon names to load.
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      array    $icons    An array of strings (icon names)
	 */
	protected $icons = array();

	/**
	 * Store a copy of the input type from the attributes.
	 *
	 * @var string
	 */
	protected $input_type = '';

	/**
	 * The user selected values
	 *
	 * @var array
	 */
	private $values = array();

	/**
	 * Saved field html attributes
	 *
	 * @var array
	 */
	private $html_attributes = array();

	/**
	 * Options for fields that have multiple choice
	 *
	 * @var array
	 */
	private $options = array();
	/**
	 * Whether or not we have already calculated the options.
	 *
	 * @var array
	 */
	private $options_init = false;

	/**
	 * Context for the field, such as admin, block-editor, elementor, etc.
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $name    Name
	 */
	protected $context = '';

	/**
	 * Context path, eg - 'edit/123'
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $name    Name
	 */
	protected $context_path = '';

	/**
	 * Minimum settings to init.
	 *
	 * @var array
	 */
	private $default_attributes = array();

	/**
	 * Extra render data to be passed to to the render function.
	 *
	 * @var array
	 */
	private $render_data = array();
	/**
	 * Array of callbacks matching the render_data keys.
	 *
	 * @var array
	 */
	private $render_escape_callbacks = array();
	/**
	 * Keeps track of used IDs to ensure we can generate unique IDs.
	 *
	 * @var array
	 */
	private static $instance_ids = array();

	/**
	 * A unique name to be used in the URL.
	 *
	 * @var string
	 */
	private $url_name = '';

	/**
	 * The CSS string for this style preset.
	 *
	 * @var string
	 */
	private $css = '';

	/**
	 * Additional data that can be used to generate the field.
	 *
	 * @var string
	 */
	private $connected_data = array();

	/**
	 * Get the status of the query
	 */
	public function get_css() {
		return $this->css;
	}
	/**
	 * Set the status of the query
	 *
	 * @param string $status The status to set.
	 */
	public function set_css( $css ) {
		$this->css = $css;
	}

	/**
	 * Overridable function for setting field defaults.
	 *
	 * @param array $defaults Default attributes.
	 *
	 * @return array New default attributes.
	 */
	/*
	 public function get_default_attributes() {
		return array(
			'dataType'  => '',
			'addClass'  => '',
			'scale'     => 2,
			'label'     => '',
			'showLabel' => 'no',
			// Stored reference, but not required.
			'queryId'   => 0,
			'name'      => '',
			'width'     => '',
			'align'     => '',
			'alignment' => '',
		);
	} */
	public function get_default_attributes() {
		// TODO - defaults should be set based on the fields settings.
		// We need to apply the `dependsOn`/ conditional logic to get the necessary defaults.
		// We should probably also "clean" the attributes before saving to remove settings/keys
		// that are not needed anymore by the field.
		$defaults = \Search_Filter\Fields\Settings::get_defaults();
		return $defaults;
	}
	/**
	 * Init the field from already loaded attributes.
	 */
	public function init() {

		parent::init();

		if ( $this->uid === 0 ) {
			$this->uid = self::get_instance_id( 'field' );
		}
		$this->refresh_query_id();

		if ( $this->url_name === '' ) {
			$this->url_name = $this->get_url_name();
		}

		// Reset class attribute.
		$this->html_attributes['class'] = '';

		// Set the field values.
		$this->values = array();

		// Attributes needs has_init to be true so we can use `get_values`.
		$this->set_html_attributes();

		// Add user defined custom classes.
		$this->add_class( $this->attributes['addClass'] );

		// Setup values from the URL.
		$this->parse_url_value();
	}
	/**
	 * Inits the data from a DB record.
	 *
	 * @param [type] $item Database record.
	 */
	public function load_record( $item ) {
		parent::load_record( $item );
		$this->set_context( $item->get_context() );
		$this->set_context_path( $item->get_context_path() );
		$this->set_css( $item->get_css() );
	}
	/**
	 * Generate the unique name to be used in the URL.
	 *
	 * @return string The name to be used in the URL
	 */
	public function get_url_name() {
		$url_name = 'field_' . self::get_instance_id( 'url_name' );
		$url_name = apply_filters( 'search-filter/field/url_name', $url_name, $this->get_attributes() );
		return $url_name;
	}
	
	/**
	 * Generates a results URL for the field.
	 * 
	 * We usually want this to be empty, but sometimes a field
	 * will want to link to something like a taxonomy archive.
	 *
	 * @return string The name to be used in the URL
	 */
	public function get_url() {
		return '';
	}
	/**
	 * Parses a value from the URL.
	 */
	public function parse_url_value() {
		$url_param_name = self::url_prefix() . $this->get_url_name();
		$values         = isset( $_GET[ $url_param_name ] ) ? urldecode_deep( wp_unslash( $_GET[ $url_param_name ] ) ) : '';

		if ( $values !== '' ) {
			$this->set_values( explode( ',', $values ) );
		}
	}

	/**
	 * Sets the value of the field.
	 *
	 * @param array $values The values to set.
	 */
	public function set_values( $values ) {
		$this->values = $values;
	}

	/**
	 * Get the ID of the field
	 */
	public function get_id() {
		if ( $this->id !== 0 ) {
			return $this->id;
		}

		// Blocks use fieldId to reference their DB equivalent.
		if ( isset( $this->attributes['fieldId'] ) ) {
			// TODO - we should probably normalise this so they all use the `id` attribute.
			return absint( $this->attributes['fieldId'] );
		}

		return $this->id;
	}

	/**
	 * Gets the WP_Query args based on the field value.
	 */
	public function apply_wp_query_args( $query_args = array() ) {
		$query_args = apply_filters( 'search-filter/field/wp_query_args', $query_args, $this->get_attributes(), $this->get_values() );
		return $query_args;
	}

	/**
	 * Sets the `attributes` var, which are used in the container markup.
	 *
	 * @since    3.0.0
	 */
	private function set_html_attributes() {

		$classes    = array();
		$base_class = 'search-filter';

		array_push( $classes, $base_class );

		$field_type = $this->attributes['type'];
		if ( $field_type === 'control' ) {
			$control_type = $this->attributes['controlType'];
			array_push( $classes, 'search-filter--control-type-' . $control_type );

		} else {
			$input_type = $this->attributes['inputType'];
			array_push( $classes, 'search-filter--input-type-' . $input_type );

		}

		array_push( $classes, 'search-filter--scale' );
		array_push( $classes, 'search-filter--scale-' . $this->attributes['scale'] );
		array_push( $classes, 'search-filter--type-' . $field_type );

		array_push( $classes, 'search-filter-field--' . $this->get_id() );
		array_push( $classes, 'search-filter--width-' . $this->attributes['width'] );
		array_push( $classes, 'search-filter--align-' . $this->attributes['align'] );
		array_push( $classes, 'search-filter--align-text-' . $this->attributes['alignment'] );
		// Add block editor class.
		if ( $this->attributes['align'] === 'wide' ) {
			array_push( $classes, 'alignwide' );
		}

		array_push( $classes, 'search-filter--style-' . $this->get_calc_styles_id() );
		// TODO: add query id class.
		$this->html_attributes['class'] = implode( ' ', $classes );

		$this->html_attributes['data-search-filter-uid'] = $this->uid;
	}

	/**
	 * Get the styles ID.  If its 0, its the default styles ID, so use that.
	 *
	 * @return void
	 */
	private function get_calc_styles_id() {
		// Add styles class, if its `0` (default) then fetch the default ID.
		$styles_id = isset( $this->attributes['stylesId'] ) ? absint( $this->attributes['stylesId'] ) : 0;
		if ( 0 === $styles_id ) {
			// TODO - need a wrapper for getting default styles.
			$styles_id = absint( Styles::get_default_styles_id() );
		}
		return $styles_id;
	}

	/**
	 * Add a class to the filter
	 *
	 * @param string $class_names Class names seperated by spaces.
	 *
	 * @since    3.0.0
	 */
	protected function add_class( $class_names ) {

		if ( empty( $class_names ) ) {
			return;
		}

		$this->html_attributes['class'] = trim( $this->html_attributes['class'] ) . ' ' . $class_names;
	}
	/**
	 * Add a html attribute to the filter - expects unescaped input
	 *
	 * @param string $attribute_name    Name of the attribute.
	 * @param string $attribute_value   Value of the attribute.
	 *
	 * @since    3.0.0
	 */
	protected function add_html_attribute( $attribute_name, $attribute_value ) {
		$this->html_attributes[ $attribute_name ] = $attribute_value;
	}
	/**
	 * Get the attributes
	 *
	 * @since    3.0.0
	 */
	public function get_html_attributes() {
		return $this->html_attributes;
	}


	/**
	 * Get the values that are set for this field
	 *
	 * @since    3.0.0
	 */
	protected function get_values() {
		return $this->values;
	}

	/**
	 * If there are any values set for this field
	 *
	 * @since    3.0.0
	 */
	protected function has_values() {
		return count( $this->get_values() ) > 0;
	}

	/**
	 * Quick way to retreive the single field value.
	 *
	 * Many field types have only one possible value.
	 *
	 * @return mixed The value of the field.
	 */
	public function get_value() {
		$values = $this->get_values();
		$value  = isset( $values[0] ) ? $values[0] : '';
		return $value;
	}
	/**
	 * Set the related data for the field.
	 *
	 * @param array $data An associative array of data.
	 * @return void
	 */
	public function update_connected_data( $key, $data ) {
		$this->connected_data[ $key ] = $data;
	}

	public function get_connected_data() {
		return $this->connected_data;
	}

	/**
	 * Display the HTML output of the filter
	 *
	 * @param string $return   Whether to echo or return the output.
	 *
	 * @since    3.0.0
	 */
	public function render( $return = false ) {

		// We don't want to modify the internal values.
		$attributes = $this->attributes;

		// Trigger action when starting.
		do_action( 'search-filter/fields/field/before_render', $attributes, $this->name );
		do_action( "search-filter/fields/{$this->name}/before_render", $attributes );

		// Update the live styles ID.
		$attributes['stylesId'] = $this->get_calc_styles_id();
		// Modify args before render.
		$attributes = apply_filters( 'search-filter/fields/field/render/attributes', $attributes, $this->name );
		$attributes = apply_filters( "search-filter/fields/{$this->name}/render/attributes", $attributes );

		Fields::register_active_field(
			array(
				'attributes'     => $attributes,
				'options'        => $this->options,
				'values'         => $this->values,
				'uid'            => $this->uid,
				'urlName'        => $this->url_name,
				'icons'          => $this->icons,
				'url'            => $this->get_url(),
				'connectedData'  => $this->connected_data,
			)
		);
		if ( $this->get_query_id() !== 0 ) {
			Fields::register_query_data( $this->get_query_id() );
		}

		ob_start();
		echo '<div ' . Util::get_attributes_html( $this->get_html_attributes() ) . '>';
		echo $this->build();
		echo '</div>';
		$output = ob_get_clean();

		// Modify output html.
		$output = apply_filters( 'search-filter/fields/field/render_output', $output, $this->name, $attributes );
		$output = apply_filters( "search-filter/fields/{$this->name}/render_output", $output, $attributes );

		if ( ! $return ) {
			echo $output;
		}

		// Trigger action when finished.
		do_action( 'search-filter/fields/field/after_render', $attributes, $this->name );
		do_action( "search-filter/fields/{$this->name}/after_render", $attributes );

		if ( $return ) {
			return $output;
		}
	}

	/**
	 * Return this fields input type
	 */
	public function get_input_type() {
		return $this->input_type;
	}

	/**
	 * Sets a fields options (for fields that have multiple choices)
	 *
	 * @param array $options Array of options.
	 */
	public function set_options( $options ) {
		$this->options_init = true;
		$this->options      = $options;
	}
	/**
	 * Gets a fields options.
	 */
	public function get_options() {
		return $this->options;
	}
	/**
	 * Gets a fields options.
	 */
	public function has_options() {
		return $this->options_init;
	}

	/**
	 * Sets data to be used in the SSR pre-render.
	 *
	 * @param array $data An associative array of data.
	 */
	public function set_render_data( $data ) {
		$this->render_data = $data;
	}
	/**
	 * Sets the callbacks to be used for escaping the render data.
	 *
	 * @param array $callbacks Associative array of callbacks names (as strings).
	 */
	public function set_render_escape_callbacks( $callbacks ) {
		$this->render_escape_callbacks = $callbacks;
	}
	/**
	 * Get the template base directory for this field.
	 *
	 * Override when extending this class to supply your own folders.
	 *
	 * @return string The template base directory.
	 */
	public function get_template_dir() {
		$field_type = $this->attributes['type'];
		return SEARCH_FILTER_PATH . 'includes' . DIRECTORY_SEPARATOR . 'templates' . DIRECTORY_SEPARATOR . $field_type;
	}

	/**
	 * Constructs the markup of the field input.
	 *
	 * This should be overriden in custom field classes.
	 *
	 * @since    3.0.0
	 *
	 * @return string The field markup.
	 */
	public function build() {
		$field_type   = $this->attributes['type'];
		$input_type   = $field_type === 'control' ? $this->attributes['controlType'] : $this->attributes['inputType'];
		$template_dir = $this->get_template_dir();

		$data            = $this->attributes;
		$data['options'] = $this->options;

		// TODO - not every label needs an ID, maybe this should be moved into the individual field classes?
		$data['labelUid']                           = self::get_instance_id( 'label' );
		$this->render_escape_callbacks['labelUid']  = 'absint';
		$this->render_escape_callbacks['label']     = 'esc_html';
		$this->render_escape_callbacks['showLabel'] = '';

		$data_raw     = wp_parse_args( $this->render_data, $data );
		$data_escaped = $this->escape_render_data( $data_raw, $this->render_escape_callbacks );

		// TODO.
		$data            = apply_filters( 'search-filter/field/build/data', $data_escaped, $this->name );
		$template_output = '';
		if ( file_exists( $template_dir . DIRECTORY_SEPARATOR . $input_type . '.php' ) ) {
			ob_start();
			include $template_dir . DIRECTORY_SEPARATOR . $input_type . '.php';
			$template_output = ob_get_clean();
		} else {
			// TODO - throw warning about extending the Field class and implementing this method.
		}
		return $template_output;
	}

	/**
	 * Escape the render data, ignore keys that don't have callbacks.
	 *
	 * @param array $data The data to be escaped.
	 * @param array $callbacks The callbacks used to escape the data.
	 *
	 * @return array The escaped data object.
	 */
	private function escape_render_data( $data, $callbacks ) {
		$escaped_data = array();
		foreach ( $data as $key => $value ) {
			if ( isset( $callbacks[ $key ] ) ) {
				$esc_callback = $callbacks[ $key ];
				if ( is_array( $value ) && is_array( $esc_callback ) ) {
					$escaped_data[ $key ] = array();
					foreach ( $value as $child ) {
						$escaped_child = $this->escape_render_data( $child, $esc_callback );
						array_push( $escaped_data[ $key ], $escaped_child );
					}
				} elseif ( $esc_callback === '' ) {
					// Then there was no callback specified.
					$escaped_data[ $key ] = $value;
				} else {
					// Run an escape callback if it exists.
					$escaped_data[ $key ] = call_user_func( $callbacks[ $key ], $data[ $key ] );
				}
			}
		}
		// Skip any keys that don't have a callback.
		return $escaped_data;
	}

	/**
	 * Sets the Field context (admin, block-editor, elementor, beaverbuilder etc)
	 *
	 * @param string $context The context to set.
	 * @param string $path The path of the context.
	 */
	public function set_context( $context ) {
		$this->context = $context;
	}
	/**
	 * Sets the Field context path - eg 'post/123'
	 *
	 * @param string $context The context to set.
	 * @param string $path The path of the context.
	 */
	public function set_context_path( $path ) {
		$this->context_path = $path;
	}
	/**
	 * Gets the query ID based on the query integration setting.
	 *
	 * @return string The URL prefix.
	 */
	public function get_query_id() {
		return $this->query_id;
	}
	public function refresh_query_id() {
		if ( isset( $this->attributes['queryId'] ) ) {
			$this->query_id = absint( $this->attributes['queryId'] );
		}
	}

	/**
	 * Saves the field
	 *
	 * @return int The saved field ID.
	 */
	public function save( $args = array() ) {
		$this->refresh_query_id();
		$this->regenerate_css();

		$extra_args = array(
			'context'      => $this->context,
			'context_path' => $this->context_path,
			'query_id'     => $this->query_id,
			'css'          => $this->get_css(),
		);
		return parent::save( $extra_args );
	}

	/**
	 * Deletes an instantiated field from the database.
	 *
	 * @return bool True if deleted, false if not.
	 */
	public function delete() {
		if ( ! current_user_can( 'manage_options' ) ) {
			return false;
		}
		if ( $this->id > 0 ) {
			$result   = self::destroy( $this->id );
			$this->id = 0;
			return $result;

		}
		return false;
	}

	/**
	 * Gets the prefix to be added to URLs to avoid collisions.
	 *
	 * @return string The URL prefix.
	 */
	public static function url_prefix() {
		// TODO.
		return '_';
	}

	/**
	 * Creates a new instance of the field class with given data.
	 *
	 * // TODO - this needs reworking - we probably want to pass a name, type and attributes at least.
	 *
	 * @param array $attributes The init field attributes.
	 * @param array $context Context data of the field, contains keys 'context' and 'path'.
	 * @return Field The new instance of the field class.
	 */
	public static function create( $attributes = array(), $context = array() ) {
		$static_class = static::class;
		$new_field    = new $static_class();
		$new_field->set_attributes( $attributes );

		if ( isset( $context['context'] ) ) {
			$new_field->set_context( $context['context'] );
		}
		if ( isset( $context['path'] ) ) {
			$new_field->set_context_path( $context['path'] );
		}
		return $new_field;
	}

	/**
	 * Process the attributes and run init local vars
	 *
	 * @param array $attributes  Field attributes.
	 *
	 * @since    3.0.0
	 */
	public function set_attributes( $attributes, $replace = false ) {
		parent::set_attributes( $attributes, $replace );
		$this->refresh_query_id();
		$this->init();
	}
	/**
	 * Creates a new instance of the field from a database record.
	 *
	 * @param StdClass $item The database record.
	 *
	 * @return Field The new instance of the field class.
	 */
	public static function create_from_record( $item ) {
		$static_class = static::class;
		$new_field    = new $static_class();
		$new_field->load_record( $item );
		return $new_field;
	}

	/**
	 * Find a field by conditions
	 *
	 * @param array  $conditions  Column name => value pairs.
	 * @param string $return  Whether to return the object or the record.
	 * @return self|Row|WP_Error
	 */
	public static function find( $conditions, $return = 'object' ) {
		$query_args = array(
			'number'  => 1, // Only retrieve a single record.
			'orderby' => 'date_published',
			'order'   => 'asc',
		);
		$query_args = wp_parse_args( $conditions, $query_args );
		/**
		 * TODO - we probably want to wrap this in our settings API
		 * so we never call the same field twice (maybe we need to
		 * update the API to support searching for fields without
		 * query ID)
		 */
		$query = new Field_Query( $query_args );

		// Bail if nothing found.
		if ( empty( $query->items ) ) {
			return new \WP_Error( 'search_filter_field_not_found', __( 'Field not found.', 'search-filter' ) );
		}

		if ( $return === 'record' ) {
			return $query->items[0];
		}

		// Create the field.
		$field = Field_Factory::create_from_record( $query->items[0] );
		return $field;
	}
	/**
	 * Generates a unique ID based on the subject.
	 *
	 * @param array $subject The subject to generate the ID for.
	 *
	 * @return int The instance ID.
	 */
	public static function get_instance_id( $subject ) {
		if ( ! isset( self::$instance_ids[ $subject ] ) ) {
			self::$instance_ids[ $subject ] = 0;
		} else {
			self::$instance_ids[ $subject ]++;
		}
		return self::$instance_ids[ $subject ];
	}


	/**
	 * Adds an icon name to the icons array.
	 *
	 * @param string $icon_name The name of the icon to add.
	 */
	public static function add_icon( $icon_name ) {
		self::$icons[] = $icon_name;
	}
	/**
	 * Gets the icons array.
	 *
	 * @return array The icons array.
	 */
	public static function get_icons() {
		return self::$icons;
	}
	/**
	 * Deletes an icon name from the icons array.
	 */
	public static function remove_icon( $icon_name ) {
		$index = array_search( $icon_name, self::$icons, true );
		if ( false !== $index ) {
			unset( self::$icons[ $index ] );
		}
	}
	/**
	 * Regenerates the CSS for the style preset.
	 *
	 * @since   3.0.0
	 */
	public function regenerate_css() {
		$this->css = $this->generate_css();
	}
	/**
	 * Generates the CSS for the style preset.
	 *
	 * @return string The generated CSS.
	 */
	private function generate_css() {
		$css = '';
		// Get the base styles class for the ID.
		$styles_class = '.search-filter-field--' . intval( $this->get_id() );

		$css .= $styles_class . '{';
		$css .= CSS_Loader::clean_css( $this->create_attributes_css( $this->get_attributes() ) );
		$css .= '}';

		return $css;
	}
	/**
	 * Parses the attributes into CSS styles (variables).
	 *
	 * @since   3.0.0
	 *
	 * @param array $attributes  The saved style attributes.
	 *
	 * @return string The generated CSS.
	 */
	public function create_attributes_css( $attributes ) {
		$mapped_css_vars = array(
			'--search-filter-label-spacing' => 'labelSpacing',
			'--search-filter-label-scale'   => 'labelScale',
		);

		$css = '';
		foreach ( $mapped_css_vars as $css_var => $attribute_key ) {
			if ( isset( $attributes[ $attribute_key ] ) && $attributes[ $attribute_key ] !== '' ) {
				$css .= $css_var . ':' . sanitize_key( $attributes[ $attribute_key ] );
				$css .= ';';
			}
		}

		$css = apply_filters( 'search-filter/styles/style/create_attributes_css', $css, $attributes );
		return $css;
	}


	public function get_attributes() {
		$attributes = parent::get_attributes();
		$attributes = apply_filters( 'search-filter/fields/field/get_attributes', $attributes, $this->get_id() );
		return $attributes;
	}
}
