<?php
/**
 * Choice Filter base class
 *
 * @link       https://searchandfilter.com
 * @since      3.0.0
 * @package    Search_Filter
 * @subpackage Search_Filter/Fields/Filter
 */

namespace Search_Filter\Fields;

use Search_Filter\Settings;
use Search_Filter\Fields\Field;
use Search_Filter\Core\WP_Data;
use Search_Filter\Fields\Data\Taxonomy_Options;
use Search_Filter\Queries\Query;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Handles things a field with choices with need - such
 * as a list of options.
 */
class Filter extends Field {

	public function get_default_attributes() {
		// TODO - defaults should be set based on the fields settings.
		// We need to apply the `dependsOn`/ conditional logic to get the necessary defaults.
		// We should probably also "clean" the attributes before saving to remove settings/keys
		// that are not needed anymore by the field.
		$defaults         = \Search_Filter\Fields\Settings::get_defaults_by_context( 'admin/field/filter' );
		$defaults['type'] = 'filter';
		return $defaults;
	}
	/**
	 * Get the list of options based on data attributes
	 *
	 * @return array
	 */
	public function get_options() {
		if ( ! parent::has_options() ) {
			$this->create_options();
		}
		return parent::get_options();
	}

	/**
	 * Get the list of options based on data attributes
	 * 
	 * TODO - we should be able to expect some of these attributes
	 * to be set rather than checking them all.
	 * 
	 * This is partly related to the create_field endpoint, it is
	 * hit while fields are being initialised and as a result the
	 * attributes are malformed.
	 * 
	 * 1. We need a reliable way to init fields with the correct
	 *    attributes & sensible defaults.
	 * 2. Resolve the settings in PHP before loading the page,
	 *    use it as the default state for the JS so we don't send
	 *    the wrong data to the server when creating new fields.
	 * 
	 *
	 * @return array
	 */
	protected function create_options() {

		if ( ! $this->has_init() ) {
			return array();
		}

		do_action( 'search-filter/fields/filter/choice/create_options/start' );

		$options = array();
		// Now check things like data type and data source, to figure out which options should be generated
		// get the options for the field - used by all choice decendants.
		if ( 'post_attribute' === $this->attributes['dataType'] ) {
			$data_source = $this->attributes['dataPostAttribute'];

			if ( 'post_type' === $data_source ) {
				if ( ! isset( $this->attributes['postTypes'] ) || empty( $this->attributes['postTypes'] ) ) {
					// If no post type is chosen, then choose all available from the query attributes.
					$this->attributes['postTypes'] = false;
					$query = Query::find( array( 'id' => $this->get_query_id() ) );
					if ( ! is_wp_error( $query ) ) {
						$this->attributes['postTypes'] = $query->get_attribute( 'postTypes' );
					}
				}

				if ( ! is_array( $this->attributes['postTypes'] ) ) {
					// TODO - throw error.
					return $options;
				}

				// TODO - lookup only the post types chosen.
				$post_types = WP_Data::get_post_types();

				foreach ( $post_types as $post_type ) {
					if ( in_array( $post_type->name, $this->attributes['postTypes'], true ) ) {
						$options[] = array(
							'value' => $post_type->name,
							'label' => html_entity_decode( $post_type->label ),
						);
					}
				}
			} elseif ( 'post_status' === $data_source ) {
				if ( empty( $this->attributes['postStati'] ) ) {
					return $options;
				}
				if ( ! is_array( $this->attributes['postStati'] ) ) {
					return $options;
				}

				$post_stati = WP_Data::get_post_stati();

				foreach ( $post_stati as $post_status ) {
					if ( in_array( $post_status->name, $this->attributes['postStati'], true ) ) {

						$options[] = array(
							'value' => $post_status->name,
							'label' => html_entity_decode( $post_status->label ),
						);
					}
				}
			}
		} elseif ( 'taxonomy' === $this->attributes['dataType'] ) {
			$data_source     = isset( $this->attributes['dataTaxonomy'] ) ? $this->attributes['dataTaxonomy'] : '';
			$hide_empty      = isset( $this->attributes['hideEmpty'] ) && $this->attributes['hideEmpty'] === 'yes' ? true : false;
			$is_hierarchical = isset( $this->attributes['taxonomyHierarchical'] ) && $this->attributes['taxonomyHierarchical'] === 'yes' ? true : false;
			$order_by        = isset( $this->attributes['taxonomyOrderBy'] ) ? $this->attributes['taxonomyOrderBy'] : '';
			$order_dir       = isset( $this->attributes['taxonomyOrderDir'] ) ? strtoupper( $this->attributes['taxonomyOrderDir'] ) : '';
			$limit_depth     = isset( $this->attributes['limitTaxonomyDepth'] ) && $this->attributes['limitTaxonomyDepth'] === 'yes' ? true : false;

			$data_tax_depth = 0;
			if ( $is_hierarchical && $limit_depth && isset( $this->attributes['taxonomyDepth'] ) ) {
				$data_tax_depth = (int) $this->attributes['taxonomyDepth'];
			}

			if ( $data_source !== '' ) {
				$args = array(
					'taxonomy'   => $data_source,
					'orderby'    => $order_by,
					'order'      => $order_dir,
					'hide_empty' => $hide_empty,
				);

				$taxonomy_terms   = WP_Data::get_terms( $args );
				$taxonomy_options = new Taxonomy_Options();
				$taxonomy_options->init( $taxonomy_terms );
				if ( $is_hierarchical ) {
					if ( $this->attributes['inputType'] === 'checkbox' || $this->attributes['inputType'] === 'radio' ) {
						$options = $taxonomy_options->get_hierarchical_term_options( 'nested', $data_tax_depth );
					} else {
						$options = $taxonomy_options->get_hierarchical_term_options( 'flat', $data_tax_depth );
					}
				} else {
					$options = $taxonomy_options->get_term_options();
				}

				$this->update_connected_data( 'taxonomyParents', $taxonomy_options->get_all_term_parents() );
				$this->update_connected_data( 'termIdentifiers', (array) $taxonomy_options->get_all_term_identifiers() );
			}
		}

		// Allow custom options.
		$options = apply_filters( 'search-filter/field/choice/options', $options, $this->attributes );

		do_action( 'search-filter/fields/filter/choice/create_options/finish' );

		$this->set_options( $options );
		return $options;
	}

	/**
	 * Calculate the hierarchy depth for parent-child relationship
	 *
	 * @param  \WP_Term $term
	 * @param  array    $term_list
	 * @return int
	 */
	protected function get_taxonomy_depth( $term, $term_list ) {
		if ( ! $term || $term->parent === 0 ) {
			return 1;
		}

		$find_item = function( $id ) use ( $term_list ) {
			foreach ( $term_list as $item ) {
				if ( $id === $item->term_id ) {
					return $item;
				}
			}
		};

		$depth_level = 1;
		$parent_id   = $term->parent;

		while ( $parent = $find_item( $parent_id ) ) {
			$depth_level++;
			$parent_id = $parent->parent;
		}

		return $depth_level;
	}

	/**
	 * Gets the URL name for the field.
	 *
	 * @return string
	 */
	public function get_url_name() {
		if ( ! $this->has_init() ) {
			return parent::get_url_name();
		}

		if ( ! isset( $this->attributes['dataType'] ) ) {
			return parent::get_url_name();
		}

		if ( 'post_attribute' === $this->attributes['dataType'] ) {
			$data_source = $this->attributes['dataPostAttribute'];
			return $data_source;
		} elseif ( 'taxonomy' === $this->attributes['dataType'] ) {
			$data_source = isset( $this->attributes['dataTaxonomy'] ) ? $this->attributes['dataTaxonomy'] : '';
			return $data_source;
		}
		return parent::get_url_name();
	}

	/**
	 * If the setting is enabled, supply a URL for the field
	 * when its a taxonomy.
	 */
	public function get_url() {
		if ( ! $this->has_init() ) {
			return parent::get_url();
		}

		if ( ! isset( $this->attributes['dataType'] ) ) {
			return parent::get_url();
		}

		if ( $this->attributes['dataType'] === 'taxonomy' ) {
			$taxonomy = isset( $this->attributes['dataTaxonomy'] ) ? $this->attributes['dataTaxonomy'] : '';
			// Get taxonomy URL.
			return self::get_term_template_link( $taxonomy );
		}
		return parent::get_url();
	}

	/**
	 * 
	 */
	private function get_term_template_link( $taxonomy ) {
		$term_names = array();
		/**
		 * TODO - keep an eye on this, we're fetching only top level terms
		 * and assuming that all hierarchical terms work the same way.
		 * 
		 * We need to make sure that if we're using a query string to set
		 * the term, then we do that for all terms in the hierarchy but if
		 * we're using pretty permalinks then we need to make sure that they
		 * are added in the correct order.
		 */
		$terms      = get_terms(
			array(
				'taxonomy'   => $taxonomy,
				'hide_empty' => false,
				'depth'      => 0,
				'number'     => 1,
			)
		);
		$term = ( count($terms) === 1 ) ? $terms[0] : null;
		if ( $term === null ) {
			return '';
		}
		$term_template_link = $this->get_term_link_template( $term );
		return $term_template_link;
	}

	/**
	 * If the setting is enabled, supply a URL for the field
	 * when its a taxonomy.
	 */
	private function get_term_link_template( $term ) {
		$taxonomy_name = $term->taxonomy;
		$term_slug = $term->slug;
		$term_id = $term->term_id;

		// is_taxonomy_hierarchical
		$term_link = get_term_link( $term, $taxonomy_name );
		$term_template_link = $term_link;
		$home_url_removed = false;
		if ( strpos( $term_template_link, home_url() ) === 0 ) {
			$term_template_link = substr( $term_template_link, strlen( home_url() ) );
			$home_url_removed   = true;
		}
		$has_permalink_structure = ! empty( get_option( 'permalink_structure' ) );
	
		$replace_part = $has_permalink_structure ? $term_slug : $term_id;
		$replace_symbol = $has_permalink_structure ? '[slug]' : '[id]';
		$term_template_link = self::string_lreplace( $replace_part, $replace_symbol, $term_template_link );

		if ( $home_url_removed === true ) {
			$term_template_link = home_url() . $term_template_link;
		}

		return $term_template_link;
   	}

	public static function sort_string_by_length( $a, $b ) {
		return strlen( $b ) - strlen( $a );
	}

	public static function string_lreplace( $search, $replace, $subject ) {
		$pos = strrpos( $subject, $search );

		if ( $pos !== false ) {
			$subject = substr_replace( $subject, $replace, $pos, strlen( $search ) );
		}

		return $subject;
	}

	/**
	 * Add the options to the json data object.
	 */
	public function get_json_data() {
		if ( ! $this->has_init() ) {
			return array();
		}
		$json_data = parent::get_json_data();

		$json_data['options'] = $this->get_options();

		if ( $this->get_attribute( 'dataType' ) === 'taxonomy' ) {
			$json_data['connectedData'] = $this->get_connected_data();
		}

		return $json_data;

	}
	/**
	 * Gets the WP_Query args based on the field value.
	 */
	public function apply_wp_query_args( $query_args = array() ) {
		if ( ! $this->has_init() ) {
			return parent::apply_wp_query_args( $query_args );
		}

		// Only set post_type if a value is selected.
		if ( ! $this->has_values() ) {
			return parent::apply_wp_query_args( $query_args );
		}

		$values = $this->get_values();

		$query_values = array();
		// Now check things like data type and data source, to figure out which part of the query should be updated.
		if ( 'post_attribute' === $this->attributes['dataType'] ) {
			$data_source = $this->attributes['dataPostAttribute'];

			if ( 'post_type' === $data_source ) {
				if ( empty( $this->attributes['postTypes'] ) ) {
					return parent::apply_wp_query_args( $query_args );
				}
				if ( ! is_array( $this->attributes['postTypes'] ) ) {
					// TODO - throw error.
					return parent::apply_wp_query_args( $query_args );
				}
				foreach ( $values as $post_type ) {
					// Make sure the post type selected is a valid option chosen in postTypes attribute.
					if ( in_array( $post_type, $this->attributes['postTypes'], true ) ) {
						$query_values[] = $post_type;
					}
				}
				$query_args['post_type'] = $query_values;

				return parent::apply_wp_query_args( $query_args );

			} elseif ( 'post_status' === $data_source ) {
				if ( empty( $this->attributes['postStati'] ) ) {
					return parent::apply_wp_query_args( $query_args );
				}
				if ( ! is_array( $this->attributes['postStati'] ) ) {
					return parent::apply_wp_query_args( $query_args );
				}

				foreach ( $values as $post_status ) {
					// Make sure the post type selected is a valid option chosen in postTypes attribute.
					if ( in_array( $post_status, $this->attributes['postStati'], true ) ) {
						$query_values[] = $post_status;
					}
				}
				$query_args['post_status'] = $query_values;
			}
		} elseif ( 'taxonomy' === $this->attributes['dataType'] ) {
			$taxonomy_name = $this->attributes['dataTaxonomy'];
			foreach ( $values as $tax_term ) {
				if ( term_exists( $tax_term, $taxonomy_name ) ) {
					$query_values[] = $tax_term;
				}
			}
			if ( empty( $query_values ) ) {
				return parent::apply_wp_query_args( $query_args );
			}
			if ( ! isset( $query_args['tax_query'] ) ) {
				$query_args['tax_query'] = array();
			}

			// TODO - figure out how to handle this in relation to other taxonomies being set
			// in the query already (ie via the loop block).
			$query_args['tax_query']['relation'] = 'AND';

			$compare_type = 'IN';
			if ( isset( $this->attributes['multipleMatchMethod'] ) ) {
				$compare_type = $this->attributes['multipleMatchMethod'] === 'all' ? 'AND' : 'IN';
			} elseif ( isset( $this->attributes['taxonomyCompare'] ) ) {
				$compare_type = $this->attributes['taxonomyCompare'];
			}
			if ( $compare_type === 'AND' ) {
				$sub_tax_query = array(
					'relation' => 'AND',
				);
				foreach ( $query_values as $value ) {
					$sub_tax_query[] = array(
						'taxonomy' => $taxonomy_name,
						'field'    => 'slug',
						'terms'    => array( $value ),
					);
				}
				$query_args['tax_query'][] = $sub_tax_query;
			} else {
				$query_args['tax_query'][] = array(
					array(
						'taxonomy' => $taxonomy_name,
						'field'    => 'slug',
						'compare'  => 'IN',
						'terms'    => $query_values,
					),
				);
			}
		}
		return parent::apply_wp_query_args( $query_args );
	}
}
