<?php
/**
 * Gutenberg Integration Class
 *
 * @link       https://searchandfilter.com
 * @since      1.0.0
 * @package    Custom_Layouts
 */

namespace Search_Filter\Integrations;

use Search_Filter\Core\Icons;
use Search_Filter\Core\Scripts;
use Search_Filter\Core\SVG_Loader;
use Search_Filter\Fields;
use Search_Filter\Fields\Field;
use Search_Filter\Fields\Field_Factory;
use Search_Filter\Integrations;
use Search_Filter\Integrations\Gutenberg\Block_Parser;
use Search_Filter\Integrations\Gutenberg\Cron;
use Search_Filter\Queries;
use Search_Filter\Queries\Query;
use Search_Filter\Fields\Settings as SettingsField;
use Search_Filter\Styles;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * All Gutenberg integration functionality
 */
class Gutenberg {

	/**
	 * Track if a query is running (so we can attach our params the WP_Query)
	 *
	 * @since    1.0.0
	 */
	private static $is_tracking_query = false;
	/**
	 * Collected data from the current query block.  We need to pass data from the
	 * parent query block (ID) into the pre_get_posts hook as well as the child
	 * post-template block.
	 *
	 * @since    1.0.0
	 */
	private static $current_query_data = array(
		'connected_queries' => array(),
	);

	/**
	 * A collection of the queried fields that are use in the current post.
	 *
	 * @var array
	 */
	private static $post_fields = array();
	/**
	 * A store of which post IDs are linked to which queries.
	 *
	 * @var array
	 */
	private static $post_queries = array();
	/**
	 * Init
	 *
	 * @since    1.0.0
	 */

	private static $screens;

	public static function init() {
		if ( ! function_exists( 'register_block_type' ) ) {
			// Gutenberg is not active.
			return;
		}
		add_action( 'search-filter/settings/init', '\\Search_Filter\\Integrations\\Gutenberg::setup', 10 );
	}

	public static function setup() {
		if ( ! Integrations::is_enabled( 'blockeditor' ) ) {
			return;
		}

		if ( is_admin() ) {
			self::$screens = new \Search_Filter\Admin\Screens();
		}

		// Init the cron tasks.
		Cron::init();

		/**
		 * Admin facing.
		 */
		// Needs a low priority to get added before editor_script. TODO.
		add_action( 'enqueue_block_editor_assets', '\\Search_Filter\\Integrations\\Gutenberg::editor_assets', 20 );
		add_action( 'init', '\\Search_Filter\\Integrations\\Gutenberg::register_blocks', 20 );
		add_action( 'block_editor_rest_api_preload_paths', '\\Search_Filter\\Integrations\\Gutenberg::preload_api_paths', 10 );

		// Update fields when blocks are updated.
		add_action( 'save_post', '\\Search_Filter\\Integrations\\Gutenberg::save_post', 20, 2 );
		// When a post is deleted, remove entries from our tables.
		add_filter( 'delete_post', '\\Search_Filter\\Integrations\\Gutenberg::delete_post', 10, 1 );

		add_action( 'rest_save_sidebar', '\\Search_Filter\\Integrations\\Gutenberg::save_sidebar', 20, 2 );
		add_action( 'rest_after_save_widget', '\\Search_Filter\\Integrations\\Gutenberg::save_widget', 20, 4 );
		add_action( 'rest_delete_widget', '\\Search_Filter\\Integrations\\Gutenberg::delete_widget', 20, 4 );

		// Add custom category to block inserter.
		add_filter( 'block_categories_all', '\\Search_Filter\\Integrations\\Gutenberg::block_categories', 10 );

		// TODO - need to handle FSE and template parts (in edit-post), eg cover the contexts - edit-post, edit-widgets, edit-site
		// Currently handling edit-post and edit-widgets contexts
		// https://github.com/WordPress/gutenberg/blob/b5582f9c0833993edee6aac288457f466568af8f/lib/experimental/class-wp-rest-block-editor-settings-controller.php

		/**
		 * Frontend facing.
		 */
		// Add support for filtering the Query Loop block.
		add_filter( 'pre_render_block', '\\Search_Filter\\Integrations\\Gutenberg::pre_render_query_block', 10, 2 );
		// Temporarily removed.
		// add_filter( 'pre_render_block', '\\Search_Filter\\Integrations\\Gutenberg::pre_render_post_template_block', 10, 2 );
		add_filter( 'render_block', '\\Search_Filter\\Integrations\\Gutenberg::render_query_block', 10, 3 );
	}
	/**
	 * Add custom category to block inserter.
	 *
	 * @param array $categories Array of block categories.
	 * @return array
	 */
	public static function block_categories( $categories ) {
		$categories[] = array(
			'slug'  => 'search-filter',
			'title' => __( 'Search & Filter', 'search-filter' ),
		);
		return $categories;
	}

	/**
	 * Add dynamic attributes the our blocks - useful when attributes (settings in our case) are added
	 * via PHP, and we need them to be added to our block registration.
	 */
	public static function add_dynamic_attributes() {
		$attributes = array(
			'search'  => array(),
			'filter'  => array(),
			'control' => array(),
		);
		$attributes = apply_filters( 'search-filter/integrations/gutenberg/add_attributes', $attributes );
		return array( 'addAttributes' => $attributes );
	}
	/**
	 * Register the stylesheets for the gutenberg editor.
	 *
	 * @since    1.0.0
	 */
	public static function register_blocks() {

		// TODO - I'm not sure instantiating the frontend class is the best way to do this.
		// I prefer to keep the frontend or script loader class as a singleton.
		$plugin_frontend = new \Search_Filter\Frontend( SEARCH_FILTER_SLUG, SEARCH_FILTER_VERSION );
		$plugin_frontend->enqueue_scripts();
		// Instantiate the frontend so we can register the styles.
		$plugin_frontend->register_styles();

		// Setup the main Gutenberg script dependencies.

		// Load our plugins for the block editor.
		wp_register_script( 'search-filter-block-editor-plugins', Scripts::get_admin_assets_url() . 'js/admin/block-editor-plugins.js', array( 'wp-plugins', 'wp-edit-post', 'wp-element', 'search-filter-gutenberg' ), SEARCH_FILTER_VERSION, false );

		$script_dependencies = array( 'search-filter', 'wp-blocks', 'wp-i18n', 'wp-element', 'wp-plugins', 'wp-edit-site' );
		wp_register_script( 'search-filter-gutenberg', Scripts::get_admin_assets_url() . 'js/admin/gutenberg.js', $script_dependencies, SEARCH_FILTER_VERSION, false );
		Scripts::attach_globals(
			'search-filter-gutenberg',
			'admin',
			self::add_dynamic_attributes()
		);

		$blocks_dir = realpath( plugin_dir_path( __FILE__ ) ) . DIRECTORY_SEPARATOR . 'gutenberg' . DIRECTORY_SEPARATOR . 'blocks' . DIRECTORY_SEPARATOR;

		register_block_type_from_metadata(
			$blocks_dir . 'search' . DIRECTORY_SEPARATOR,
			array(
				'editor_script'   => 'search-filter-gutenberg',
				'style_handles'   => array( 'search-filter' ),
				'render_callback' => array( 'Search_Filter\\Integrations\\Gutenberg', 'render_search_field' ),
			)
		);
		register_block_type_from_metadata(
			$blocks_dir . 'filter' . DIRECTORY_SEPARATOR,
			array(
				'editor_script'   => 'search-filter-gutenberg',
				'style_handles'   => array( 'search-filter' ),
				'render_callback' => array( 'Search_Filter\\Integrations\\Gutenberg', 'render_filter_field' ),
			)
		);
		register_block_type_from_metadata(
			$blocks_dir . 'control' . DIRECTORY_SEPARATOR,
			array(
				'editor_script'   => 'search-filter-gutenberg',
				'style_handles'   => array( 'search-filter' ),
				'render_callback' => array( 'Search_Filter\\Integrations\\Gutenberg', 'render_control_field' ),
			)
		);
		register_block_type_from_metadata(
			$blocks_dir . 'reusable' . DIRECTORY_SEPARATOR,
			array(
				'editor_script'   => 'search-filter-gutenberg',
				'style_handles'   => array( 'search-filter' ),
				'render_callback' => array( 'Search_Filter\\Integrations\\Gutenberg', 'render_reusable_field' ),
			)
		);
	}
	/**
	 * Adds our commonly used (required on init) rest api paths for blocks
	 *
	 * @param array $preload_paths  Existing api paths.
	 * @return array
	 */
	public static function preload_api_paths( $preload_paths ) {
		if ( self::$screens->is_search_filter_screen() ) {
			// If we're on one of our admin screens, then we don't need to add this to the preload.
			return;
		}
		$default_style_id = Styles::get_default_styles_id();
		$paths            = array(
			'/search-filter/v1/queries',
			'/search-filter/v1/styles',
			'/search-filter/v1/admin/styles/default',
			'/search-filter/v1/records/styles/' . $default_style_id,
			'/search-filter/v1/admin/settings?section=queries',
			'/search-filter/v1/admin/settings?section=fields',
			'/search-filter/v1/admin/settings?section=styles',
		);

		global $pagenow;
		global $post_id;

		$context_path = '';
		if ( $pagenow === 'widgets.php' ) {
			$context_path = 'widgets';
		} elseif ( isset( $post_id ) ) {
			$context_path = 'post/' . $post_id;
		}
		// Preload context field IDs so we can track copy + pasted fields from other contexts.
		$paths[] = '/search-filter/v1/fields/context/ids?context=block-editor&context_path=' . $context_path;

		$preload_paths = array_merge( $preload_paths, $paths );
		return $preload_paths;
	}
	/**
	 * Display inline footer scripts and SVGs
	 *
	 * @return void
	 */
	public static function footer_scripts() {
		SVG_Loader::output();
	}
	/**
	 * Renders a fields html using block attributes.
	 *
	 * @param array $block_attributes  The saved block attributes.
	 * @return string
	 */
	public static function render_field( $block_attributes ) {
		$args = array();
		$args = wp_parse_args( $args, $block_attributes );

		ob_start();
		try {
			$field = Field_Factory::create( $block_attributes );
		} catch ( \Exception $e ) {
			echo esc_html( $e->getMessage() );
		}
		if ( $field ) {
			$field->render();
		}
		$output = ob_get_clean();
		return $output;
	}
	/**
	 * Renders a search field.
	 *
	 * @param array $block_attributes  The saved block attributes.
	 * @return string
	 */
	public static function render_search_field( $block_attributes ) {
		$block_attributes['type']      = 'search';
		$block_attributes['inputType'] = 'text';
		return self::render_field( $block_attributes );
	}
	/**
	 * Renders a filter field.
	 *
	 * @param array $block_attributes  The saved block attributes.
	 * @return string
	 */
	public static function render_filter_field( $block_attributes ) {
		$block_attributes['type'] = 'filter';
		return self::render_field( $block_attributes );
	}
	/**
	 * Renders a control field.
	 *
	 * @param array $block_attributes  The saved block attributes.
	 * @return string
	 */
	public static function render_control_field( $block_attributes ) {
		$block_attributes['type'] = 'control';
		return self::render_field( $block_attributes );
	}
	/**
	 * Renders a reusable field.
	 *
	 * @param array $block_attributes  The saved block attributes.
	 * @return string
	 */
	public static function render_reusable_field( $block_attributes ) {
		$field_id = absint( $block_attributes['fieldId'] );
		$field    = Field::find( array( 'id' => $field_id ) );
		if ( is_wp_error( $field ) ) {
			if ( current_user_can( 'manage_options' ) ) {
				return '<em>' . __( 'Field not found', 'search-filter' ) . '</em>';
			}
			return '';
		}
		return $field->render( true );
	}

	/**
	 * Load assets required for the block editor.
	 *
	 * @return void
	 */
	public static function editor_assets() {
		if ( self::$screens->is_search_filter_screen() ) {
			// If we're on one of our admin screens, then we don't need to load the assets.
			// For some reason, using some FSE / block editor themes, the `enqueue_block_editor_assets`
			// hook is called on our screens.
			return;
		}

		Icons::load();

		$js_file_ext  = '.js';
		$css_file_ext = '.css';

		wp_enqueue_style( 'search-filter-gutenberg', Scripts::get_admin_assets_url() . 'css/admin/gutenberg' . $css_file_ext, array( 'wp-components' ), SEARCH_FILTER_VERSION, 'all' );
		wp_enqueue_script( 'search-filter-block-editor-plugins' );
		// Add our inline JS and SVGs.
		add_action( 'admin_print_footer_scripts', 'Search_Filter\\Integrations\\Gutenberg::footer_scripts', 1 );
	}

	/**
	 * Formats our settings into block attributes
	 * // TODO - Currently not in use, but maybe we should take this approach again
	 *
	 * @return array
	 */
	public static function get_block_attributes_json( $type ) {

		$fields_settings = SettingsField::get_data();

		$context    = 'block/field/' . $type;
		$attributes = array();
		// Loop through the settings object.
		foreach ( $fields_settings as $fields_setting ) {
			// Ensure that type + context are set.
			if ( isset( $fields_setting['type'] ) && isset( $fields_setting['context'] ) && is_array( $fields_setting['context'] ) ) {
				// if the setting has a matching context then add it to the block attributes
				if ( in_array( $context, $fields_setting['context'], true ) ) {

					if ( $fields_setting['type'] !== 'slot' ) {
						$default   = isset( $fields_setting['default'] ) ? $fields_setting['default'] : '';
						$attribute = array(
							'default' => $default,
							'type'    => $fields_setting['type'],
						);
						if ( isset( $fields_setting['items'] ) ) {
							$attribute['items'] = $fields_setting['items'];
						}
						$attributes[ $fields_setting['name'] ] = $attribute;
					}
				}
			}
		}
		// Add fieldId attribute to the block.
		$attributes['fieldId']   = array(
			'default' => '',
			'type'    => 'string',
		);
		$attributes['alignment'] = array(
			'default' => '',
			'type'    => 'string',
		);
		return $attributes;
	}

	public static function get_post_field_records( $post_id ) {
		$query_args                  = array();
		$query_args['context']       = 'block-editor';
		$query_args['context_path']  = 'post/' . $post_id;
		$block_editor_fields_records = Fields::find( $query_args, 'records' );
		return $block_editor_fields_records;
	}
	/**
	 * Detects when our blocks are saved and stores the settings in our fields table.
	 */
	public static function save_post( $post_id, $post ) {

		if ( wp_is_post_revision( $post ) ) {
			return;
		}
		if ( wp_is_post_autosave( $post ) ) {
			return;
		}

		$status = 'disabled';
		if ( $post->post_status === 'publish' ) {
			$status = 'enabled';
		}

		$updated_field_ids = array();
		if ( has_blocks( $post ) ) {
			// Check for any saved blocks and update their attributes.
			$updated_field_ids = self::update_fields_from_blocks_content( $post->post_content, $status );
		}

		// TODO - although not strictly necessary right now, we probably want to add IDs for fields that
		// don't have fieldId, usually this would be considered an error as one should be generated when
		// adding a post, but in the future it may be possible to create block editor posts server side,
		// and doing this would support that use case.

		// We also need to check our DB for any fields that are no longer in use and delete them.
		// TODO - check how this is working with the site editor updates.
		$post_field_records = self::get_post_field_records( $post_id );

		foreach ( $post_field_records as $item ) {
			// If there are fields assigned to this post, but no longer connected to any blocks
			// then we need to delete them.
			if ( ! in_array( $item->get_id(), $updated_field_ids, true ) ) {
				Field::destroy( $item->get_id() );
			}
		}
	}

	/**
	 * Returns an array containing the references of
	 * the passed blocks and their inner blocks.
	 * Taken from wp-includes/block-template-utils.phjp
	 *
	 * @since 5.9.0
	 * @access private
	 *
	 * @param array $blocks array of blocks.
	 *
	 * @return array block references to the passed blocks and their inner blocks.
	 */
	public static function flatten_blocks( &$blocks ) {
		$all_blocks = array();
		$queue      = array();
		foreach ( $blocks as &$block ) {
			$queue[] = &$block;
		}

		while ( count( $queue ) > 0 ) {
			$block = &$queue[0];
			array_shift( $queue );
			$all_blocks[] = &$block;

			if ( ! empty( $block['innerBlocks'] ) ) {
				foreach ( $block['innerBlocks'] as &$inner_block ) {
					$queue[] = &$inner_block;
				}
			}
		}

		return $all_blocks;
	}
	public static function update_fields_from_blocks_content( $content, $status = '' ) {
		// Check for any saved blocks and update their attributes.
		$blocks = parse_blocks( $content );
		// TODO - maybe we should just parse this ourselves (the code for this is already on
		// github), and only look for our blocks.
		// Would surely be a faster regex as we don't care about the rest, or if blocks
		// are nested.

		$blocks              = self::flatten_blocks( $blocks );
		$block_types_to_find = array(
			'search-filter/search',
			'search-filter/filter',
			'search-filter/control',
		);
		// Track which fields are in use.
		$updated_field_ids = array();
		foreach ( $blocks as $block ) {
			if ( isset( $block['blockName'] ) && in_array( $block['blockName'], $block_types_to_find ) ) {
				$attributes = $block['attrs'];
				// Update saved field attributes based on block attributes.
				if ( isset( $attributes['fieldId'] ) ) {
					$field_id = $attributes['fieldId'];

					// TODO
					// - check if field_context is valid.
					$updated_field_ids[] = absint( $field_id );
					$field_record        = Field::find( array( 'id' => $field_id ), 'record' );
					if ( ! is_wp_error( $field_record ) ) {
						$field = Field::create_from_record( $field_record );
						$field->set_attributes( $attributes );
						if ( ! empty( $status ) ) {
							$field->set_status( $status );
						}
						$field->save();
					}
				} else {
					// TODO - error - there shouldn't be a block without a fieldId.
				}
			}
		}

		return $updated_field_ids;
	}
	public function delete_fields_from_context( $context ) {

	}
	public static function save_widget( $id, $existing_sidebar_id, $request, $creating ) {
		if ( ! isset( $request['instance'] ) ) {
			return;
		}
		if ( ! isset( $request['instance']['raw'] ) ) {
			return;
		}
		if ( ! isset( $request['instance']['raw']['content'] ) ) {
			return;
		}

		$widget_content = $request['instance']['raw']['content'];

		// Ensure content has blocks.
		if ( ! has_blocks( $widget_content ) ) {
			return;
		}

		// Check for any saved blocks and update their attributes.
		// Assume the existing sidebar ID is valid.
		$sidebar_id = $existing_sidebar_id;
		// If sidebar in request is empty, the widget changed sidebar.
		if ( ! empty( $request['sidebar'] ) ) {
			$sidebar_id = $request['sidebar'];
		}
		// Set any widgets in the inactive sidebar to disabled.
		$status = 'enabled';
		if ( $sidebar_id === 'wp_inactive_widgets' ) {
			$status = 'disabled';
		}

		$updated_field_ids = self::update_fields_from_blocks_content( $widget_content, $status );
		foreach ( $updated_field_ids as $field_id ) {
			$update_result = Field::update_meta( $field_id, 'widget_id', $id );
		}
	}
	public static function delete_widget( $widget_id, $sidebar_id, $response, $request ) {
		// Lookup field with this widget ID.
		$query_args            = array(
			'meta_query' => array(
				array(
					'key'     => 'widget_id',
					'value'   => $widget_id,
					'compare' => '=',
				),
			),
		);
		$widget_fields_records = Fields::find( $query_args, 'records' );

		if ( count( $widget_fields_records ) === 0 ) {
			return;
		}

		// Delete the field.
		foreach ( $widget_fields_records as $item ) {
			Field::destroy( $item->get_id() );
		}
	}
	public static function save_sidebar( $sidebar, $request ) {
		/**
		 * We need to do an additional check to see if any of our fields
		 * are in the inactive sidebar, and if they are, then we need to make
		 * sure they are disabled.
		 *
		 * This only applies to the new block editor powered widget screen.
		 *
		 * This covers the unusual use case, where removing a saved widget
		 * (block) puts it in the inactive widgets (sometimes), but you can
		 * only see this after refreshing the widgets screeen, after_save_widget
		 * is not fired for some reason.
		 *
		 * Steps to reproduce:
		 * 1. Ensure inactive sidebar is closed (not 100% this needs to be done)
		 * 2. Create a new field block in a widget sidebar (not in inactive sidebar)
		 * 3. Update some attribute in the block (label for example)
		 * 4. Press Update
		 * 5. From the dropdown menu (in the block toolbar) choose "Remove...[name]"
		 * 6. Press Update.
		 * 7. Notice inactive widgets is still empty
		 * 8. Refresh the widgets screen
		 * 9. Notice inactive widgets is now populated with the field block.
		 */

		$sidebars_widgets = wp_get_sidebars_widgets();
		// $sidebars_widgets = get_option( 'sidebars_widgets', array() );

		// We only want to track the widgets that got moved into inactive in this weird way...
		if ( ! isset( $sidebars_widgets['wp_inactive_widgets'] ) ) {
			return;
		}
		$inactive_widget_ids = $sidebars_widgets['wp_inactive_widgets'];

		// Lookup fields with that are associated with widgets.
		$query_args            = array(
			'meta_query' => array(
				array(
					'key'     => 'widget_id',
					'compare' => 'EXISTS',
				),
			),
		);
		$widget_fields_records = Fields::find( $query_args, 'records' );
		if ( count( $widget_fields_records ) === 0 ) {
			return;
		}
		foreach ( $widget_fields_records as $item ) {
			$widget_id = Field::get_meta( $item->get_id(), 'widget_id', true );
			if ( in_array( $widget_id, $inactive_widget_ids, true ) ) {
				// We found a field that should be disabled.
				if ( $item->get_status() === 'enabled' ) {
					// Then we need to disable it...
					$field = Field_Factory::create_from_record( $item );
					$field->set_status( 'disabled' );
					$field->save();
				}
			}
		}
	}
	private static function post_has_fields( $post_id ) {

		$post_fields = self::get_post_content_fields( $post_id );
		// Use post fields first.
		if ( count( $post_fields ) > 0 ) {
			return true;
		}
		return false;
	}

	public static function get_post_content_fields( $post_id ) {
		// If we've already looked up the fields, then just return them.
		if ( isset( self::$post_fields[ $post_id ] ) ) {
			return self::$post_fields[ $post_id ];
		}

		$post = get_post( $post_id );

		if ( $post === null ) {
			return array();
		}

		$content = $post->post_content;
		// Extract the fields on the page.
		$post_blocks = Block_Parser::extract_blocks( $content, 'array' );
		$blocks      = array();
		// Check to see if a block doesn't have a `queryId`
		// Then it will be a reusable block - so lookup its query_id.
		// TODO - probably needs optimizing for re-usable fields - this is only necessary
		// for re-usable fields that are connected to a "dynamic" query on this page.
		// Essentially the field needs to declare that it is connected to a query, so we know
		// the query should be affecting this single post/page.
		foreach ( $post_blocks as $post_block ) {
			if ( ! isset( $post_block['queryId'] ) ) {
				$field = Field_Factory::create_from_id( $post_block['fieldId'] );
				if ( ! is_wp_error( $field ) ) {
					$post_block['queryId'] = $field->get_query_id();
				}
			}
			$blocks[] = $post_block;
		}

		self::$post_fields[ $post_id ] = $blocks;
		return self::$post_fields[ $post_id ];
	}
	private static function get_post_queries( $post_id ) {
		// If we've already looked up the fields, then just return them.
		if ( isset( self::$post_queries[ $post_id ] ) ) {
			return self::$post_queries[ $post_id ];
		}

		$post_queries = Queries::find(
			array(
				'integration' => 'single/' . $post_id,
			)
		);

		self::$post_queries[ $post_id ] = $post_queries;
		return self::$post_queries[ $post_id ];
	}

	/**
	 * Find a query block that should be filtered
	 *
	 * @param [type] $should_short
	 * @param [type] $block
	 * @return void
	 */
	public static function pre_render_query_block( $should_short, $block ) {

		if ( $block['blockName'] !== 'core/query' ) {
			return $should_short;
		}
		if ( isset( $block['attrs']['namespace'] ) && $block['attrs']['namespace'] !== '' ) {
			return $should_short;
		}
		self::try_connect_to_query_loop( $block );

		return $should_short;
	}

	public static function try_connect_to_query_loop( $block, $query_integration_name = 'query_block' ) {
		if ( $block['blockName'] !== 'core/query' ) {
			return false;
		}

		// Return early if we're already tracking a query (don't allow query blocks inside of query blocks).
		if ( self::$is_tracking_query ) {
			return false;
		}

		if ( ! is_singular() ) {
			// This method only supports query loops on singular pages
			// We have a different implemenation for accessing the archives.
			return false;
		}

		// Get current post ID.
		$post_id = get_queried_object_id();

		$post_queries   = self::get_post_queries( $post_id );
		$post_query_ids = array_map(
			function( $query ) {
				return $query->get_id();
			},
			$post_queries
		);
		$post_fields    = self::get_post_content_fields( $post_id ); // Get the fields embedded in the post.

		if ( count( $post_query_ids ) === 0 && count( $post_fields ) === 0 ) {
			return false;
		}

		self::$is_tracking_query = true;
		$attributes              = $block['attrs'];
		$query_search_filter_id  = isset( $attributes['searchFilterQueryId'] ) ? $attributes['searchFilterQueryId'] : '';

		// Lookup any fields on this post, so we can check if they should affect the query or not.
		// TODO - we should also lookup any queries (saved) that are connected to this post as we
		// can have admin queries + fields affecting query blocks.
		$connected_queries    = array();
		$field_post_query_ids = array();
		// Loop through the fields once and setup the data we need.
		foreach ( $post_fields as $field ) {
			$field_post_query_ids[] = $field['queryId'];
		}

		$related_query_ids = array_unique( array_merge( $post_query_ids, $field_post_query_ids ) );

		// Loop through the fields and check if they should affect the query.
		foreach ( $related_query_ids as $query_id ) {
			// If there are fields assigned to this post, but no longer connected to any blocks
			// then we need to delete them.
			$query = null;

			try {
				$query = new Query( $query_id );
			} catch ( \Exception $e ) {
				continue;
			}

			// Now we need to check, if the query is set to "single".
			$integration_type = $query->get_attribute( 'integrationType' );
			if ( $integration_type !== 'single' ) {
				continue;
			}

			// Then check to make sure it is set to the current post ID, or "dynamic".
			$single_location = $query->get_attribute( 'singleLocation' );
			if ( ( absint( $single_location ) !== absint( $post_id ) ) && $single_location !== 'dynamic' ) {
				continue;
			}

			$single_integration = $query->get_attribute( 'singleIntegration' );
			if ( $single_integration !== $query_integration_name ) {
				continue;
			}

			$query_loop_id = $query->get_attribute( 'queryLoopId' );
			/**
			 * Check if a field should affect the query.
			 *
			 * If a field doesn't have a query loop ID, then it will auto detect any on this post,
			 * so it should affect the loop.
			 *
			 * If field does have an ID, it needs to match the query loop ID in order to affect
			 * the loop.
			 */
			if ( $query_loop_id === 'dynamic' || $query_loop_id === $query_search_filter_id ) {
				$connected_queries[] = $query;

			}
		}
		self::$current_query_data = array(
			'connected_queries' => $connected_queries,
		);

		if ( count( $connected_queries ) > 0 ) {
			// TODO - switch from using `pre_get_posts` to `query_loop_block_query_vars`?
			add_action( 'pre_get_posts', '\\Search_Filter\\Integrations\\Gutenberg::filter_query_block_query', 20 );
		}

	}
	/**
	 * Hook in our pre_get_posts hook at the point of the post template.
	 * Doing any other time will catch some custom WP_Query's that are used
	 * for the pagination blocks (hopefully they will be removed in the future).
	 *
	 * Edit: maybe we do want to access the queries for the pagination... but this
	 * is going to get expensive quickly.  For now removing this.
	 *
	 * @param [type] $should_short
	 * @param [type] $block
	 * @return void
	 */
	public static function pre_render_post_template_block( $should_short, $block ) {
		if ( $block['blockName'] !== 'core/post-template' ) {
			return $should_short;
		}

		// Return early if we're already tracking a query (don't allow query blocks inside of query blocks).
		if ( self::$is_tracking_query ) {
			add_action( 'pre_get_posts', '\\Search_Filter\\Integrations\\Gutenberg::filter_query_block_query', 10 );
		}

		return $should_short;
	}

	/**
	 * Unhook our pre_get_posts hook after the post template block.
	 *
	 * @param string         $block_content The block content.
	 * @param array          $block The full block, including name and attributes.
	 * @param \WP_Block|null $instance The block instance.
	 * @return string
	 */
	public static function render_query_block( $block_content, $block, $instance = null ) {
		if ( $block['blockName'] !== 'core/query' ) {
			return $block_content;
		}
		// We need this to run so we can unload our hooks, and modify output if necessary.

		// Namespace is used for block variation, which we probably don't want to support by default.
		/*
		 if ( isset( $block['attrs']['namespace'] ) ) {
			return $block_content;
		} */

		if ( ! self::$is_tracking_query ) {
			return $block_content;
		}

		// Reset hooks and associated data.
		remove_action( 'pre_get_posts', '\\Search_Filter\\Integrations\\Gutenberg::filter_query_block_query', 10 );

		$block_content = apply_filters( 'search-filter/integrations/gutenberg/render_query_block', $block_content, $block, self::$current_query_data['connected_queries'] );

		self::$is_tracking_query  = false;
		self::$current_query_data = array(
			'connected_queries' => array(),
		);

		return $block_content;
	}

	public static function filter_query_block_query( $query ) {
		$connected_queries = self::$current_query_data['connected_queries'];
		$query->set( 'search_filter_queries', $connected_queries );
	}

	/**
	 * Remove fields from our tables when a post is deleted.
	 *
	 * @param number $post_id The post ID.
	 * @return void
	 */
	public static function delete_post( $post_id ) {
		if ( wp_is_post_revision( $post_id ) ) {
			return;
		}
		if ( wp_is_post_autosave( $post_id ) ) {
			return;
		}
		$post_field_records = self::get_post_field_records( $post_id );

		foreach ( $post_field_records as $item ) {
			Field::destroy( $item->get_id() );
		}
	}
}
