<?php
/**
 * Class for handling the frontend display of a field.
 *
 * @link       http://searchandfilter.com
 * @since      3.0.0
 *
 * @package    Search_Filter
 * @subpackage Search_Filter/public
 */

namespace Search_Filter\Queries;

use Search_Filter\Core\CSS_Loader;
use Search_Filter\Fields;
use Search_Filter\Record_Base;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * The base field all queries.
 */
class Query extends Record_Base {
	/**
	 * The record store name
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      int    $id    ID
	 */
	public $record_store = 'query';
	/**
	 * The meta type for the meta table.
	 *
	 * @var string
	 */
	public static $meta_table = 'search_filter_query';
	/**
	 * The full string of the class name of the query class for this section.
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $records_class    The string class name.
	 */
	public static $records_class = 'Search_Filter\\Database\\Queries\\Queries';

	/**
	 * The class name to handle interacting with the record stores.
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $base_class    ID
	 */
	public static $base_class = 'Search_Filter\\Queries';
	/**
	 * Context for the query, such as single, archive, etc.
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $name    Name
	 */
	protected $context = '';
	/**
	 * The CSS string for this style preset.
	 *
	 * @var string
	 */
	private $css = '';
	/**
	 * Integration path, eg - 'archive/post'
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $name    Name
	 */
	protected $integration = '';
	/**
	 * Overridable function for setting query defaults.
	 *
	 * @param array $defaults Default attributes.
	 *
	 * @return array New default attributes.
	 */
	public function get_default_attributes() {
		$defaults = \Search_Filter\Queries\Settings::get_defaults();
		return $defaults;
	}
	/**
	 * Inits the data from a DB record.
	 *
	 * @param [type] $item Database record.
	 */
	public function load_record( $item ) {
		parent::load_record( $item );
		$this->set_context( $item->get_context() );
		$this->set_integration( $item->get_integration() );
	}
	/**
	 * Process the attributes and run init local vars
	 *
	 * @param array $attributes  Field attributes.
	 *
	 * @since    3.0.0
	 */
	public function set_attributes( $attributes, $replace = false ) {
		parent::set_attributes( $attributes, $replace );
		$this->generate_context();
		// TODO - maybe we need to move init() out of here.
		$this->init();
	}
	/**
	 * Get the settings related to the display of the results, based on
	 * query parameters.
	 * 
	 * TODO - we need to rethink this function, for taxonomy terms we're checking
	 * if we're in the frontend or not (for dynamic archive urls) - its won't be
	 * flexible enough going forward.
	 */
	public function get_results_data( $is_admin = false ) {
		$results_data = array(
			'url'   => $this->get_results_url(),
			'label' => '',
		);

		if ( ! isset( $this->attributes['integrationType'] ) ) {
			return $results_data;
		}

		$integration_type = $this->attributes['integrationType'];
		$error            = false; // TODO - lets use some more specific messaging.
		switch ( $integration_type ) {
			case 'basic':
				$results_data['label'] = '*Not available - fields will allow users to jump between archives in your site.';
				break;
			case 'results_page':
				$post_id = $this->attributes['post_id'];
				if ( 0 === $post_id ) {
					$results_data['label'] = __( 'Choose a Post/Page first', 'search-filter' );
				}
				break;
				
			case 'archive':
				if ( ! isset( $this->attributes['archiveType'] ) ) {
					break;
				}
				$archive_type = $this->attributes['archiveType'];
				if ( 'post_type' === $archive_type ) {
					$post_type = $this->attributes['postType'];
					$archive_link = get_post_type_archive_link( $post_type );
					if ( $archive_link ) {
						$results_data['url'] = $archive_link;
					} else {
						$error = true;
					}
				} elseif ( 'taxonomy' === $archive_type && isset( $this->attributes['taxonomy'] ) ) {
					$taxonomy  = $this->attributes['taxonomy'];
					// Otherwise, try to generate a term link to get the base URL.
					$tax_terms = get_terms(
						$taxonomy,
						array(
							'hide_empty' => false,
							'parent'     => 0,
						)
					);
					if ( $tax_terms ) {
						// Pick any term.
						$term      = $tax_terms[0];
						$term_link = get_term_link( $term->term_id );

						if ( ! is_wp_error( $term_link ) ) {
							$results_data['url'] = $term_link;

							$term_value = $term->slug;
							if ( ( ! get_option( 'permalink_structure' ) ) && ( 'category' === $term->taxonomy ) ) {
								/**
								 * Permalinks are disabled, in this case, check to see if the taxonomy is a category
								 * because category is an exception to other taxonomies and uses `cat=ID` in the URL
								 */
								$term_value = $term->term_id;
							}
							$results_data['label'] = rtrim( untrailingslashit( $results_data['url'] ), $term_value ) . '...';
						} else {
							$error = true;
						}
					} else {
						$error = true;
					}

					if ( $error ) {
						$results_data['label'] = __( 'Unable to get link - try adding some terms first.', 'search-filter' );
					}
					break;
				}
				break;
			case 'search':
				break;
			case 'single':
				$location  = isset( $this->attributes['singleLocation'] ) ? $this->attributes['singleLocation'] : false;
				$permalink = false;
				if ( $location && $location !== '' ) {
					$permalink = get_permalink( $location );
				}
				if ( $permalink ) {
					$results_data['url'] = $permalink;
				} else {
					$results_data['label'] = __( 'Unable to get link.', 'search-filter' );
				}
				break;
		}
		// TODO - rename according to the correct convention (its also used in the WC integration).
		$results_data = apply_filters( 'search-filter/queries/query/get_results_data', $results_data, $this->attributes );
		return $results_data;
	}

	public function get_results_url() {
		// TODO - rename according to the correct convention (its also used in the WC integration).
		$override_url = apply_filters( 'search-filter/queries/query/get_results_url/override_url', false, $this->attributes );

		if ( $override_url ) {
			return $override_url;
		}

		$integration_type = $this->attributes['integrationType'];
		switch ( $integration_type ) {
			case 'basic':
				break;
			case 'results_page':
				$post_id = $this->attributes['post_id'];
				if ( 0 !== $post_id ) {
					return get_permalink( $post_id );
				}
				break;
			case 'archive':
				if ( ! isset( $this->attributes['archiveType'] ) ) {
					return '';
					break;
				}
				$archive_type = $this->attributes['archiveType'];
				if ( 'post_type' === $archive_type ) {
					$post_type = $this->attributes['postType'];
					$archive_filter_taxonomies = isset( $this->attributes['archiveFilterTaxonomies'] ) ? $this->attributes['archiveFilterTaxonomies'] : 'no';

					global $wp_query;
					if ( $archive_filter_taxonomies === 'yes' && $wp_query->is_archive() && $wp_query->is_tax() ) {
						$queried_object = get_queried_object();
						// Get the postType taxonomies
						$taxonomies = get_object_taxonomies( $post_type );
						foreach ( $taxonomies as $taxonomy ) {
							if ( $queried_object->taxonomy === $taxonomy ) {
								return get_term_link( $queried_object->term_id );
							}
						}
					}
					
					return get_post_type_archive_link( $post_type );

				} elseif ( 'taxonomy' === $archive_type && isset( $this->attributes['taxonomy'] ) ) {
					$taxonomy  = $this->attributes['taxonomy'];
					$queried_object = get_queried_object();
					if ( isset( $queried_object->taxonomy ) && $queried_object->taxonomy === $taxonomy ) {
						return get_term_link( $queried_object->term_id );
						break;
					}
					break;
				}
				break;
			case 'search':
				return add_query_arg( 's', '', trailingslashit( get_home_url() ) );
				break;
			case 'single':
				$location  = isset( $this->attributes['singleLocation'] ) ? $this->attributes['singleLocation'] : false;
				$permalink = false;
				if ( $location === 'dynamic' && ! is_admin() && is_singular() ) {
					global $post;
					$permalink = get_permalink( $post->ID );
				} elseif ( $location !== '' ) {
					$permalink = get_permalink( $location );
				}
				if ( $permalink ) {
					return $permalink;
				}
				break;
		}
		return '';
	}

	public function apply_wp_query_args( $query_args = array() ) {
		$query_map = array(
			'post_type'      => 'postTypes',
			'post_status'    => 'postStatus',
			'posts_per_page' => 'postsPerPage',
			'sort_order'     => 'sortOrder',
			'tax_query'      => 'taxQuery',
			'sticky_posts'      => 'stickyPosts',

		);
		$callback_map = array(
			'sort_order' => 'get_sort_order_args',
			'tax_query'  => 'get_tax_query_args',
			'sticky_posts'  => 'get_sticky_posts_query_args',
		);
		foreach ( $query_map as $key => $data_key ) {
			if ( isset( $callback_map[ $key ] ) ) {
				if ( method_exists( $this, $callback_map[ $key ] ) ) {
					$new_args   = call_user_func( array( $this, $callback_map[ $key ] ), $this->attributes );
					$query_args = wp_parse_args( $new_args, $query_args );

				}
			} else {
				$query_args[ $key ] = $this->attributes[ $data_key ];
			}
		}

		$query_args = apply_filters( 'search-filter/queries/query/apply_wp_query_args', $query_args, $this->attributes );

		return $query_args;
	}

	public function get_sticky_posts_query_args( $attributes ) {
		if ( ! isset( $attributes['stickyPosts'] ) ) {
			return array();
		}

		// Don't apply the tax query if the query is a tax query archive.
		$sticky_posts  = $attributes['stickyPosts'];
		if ( $sticky_posts === 'default' ) {
			return array(
				'ignore_sticky_posts' => false
			);
		} elseif ( $sticky_posts === 'ignore' ) {
			return array(
				'ignore_sticky_posts' => true
			);
		} elseif ( $sticky_posts === 'exclude' ) {
			return array(
				'ignore_sticky_posts' => true,
				'post__not_in' => get_option( 'sticky_posts' ),
			);
		} elseif ( $sticky_posts === 'only' ) {
			return array(
				'ignore_sticky_posts' => true,
				'post__in' => get_option( 'sticky_posts' ),
			);
		}
		
		return array();
	}
	public function get_tax_query_args( $attributes ) {
		if ( ! isset( $attributes['taxonomyQuery'] ) ) {
			return array();
		}

		// Don't apply the tax query if the query is a tax query archive.
		$integration_type = $attributes['integrationType'];
		$archive_type     = $attributes['archiveType'];

		if ( $integration_type === 'archive' && $archive_type === 'taxonomy' ) {
			return array();
		}

		$tax_query = $attributes['taxonomyQuery'];

		if ( $tax_query === '' || count( $tax_query ) === 0 ) {
			return array();
		}

		$query = array(
			'tax_query' => array(
				'relation' => 'AND',
			),
		);

		// $tax_query is an array with the following structure:
		// array( 'taxonomy', 'terms', 'operator', 'includeChildren' )

		foreach ( $tax_query as $tax_query_item ) {
			$tax_query_item = array_filter( $tax_query_item );
			if ( ! empty( $tax_query_item ) ) {
				$new_item = array(
					'taxonomy' => $tax_query_item['taxonomy'],
					'operator' => $tax_query_item['operator'],
					'field'    => 'slug',
				);
				if ( isset( $tax_query_item['terms'] ) ) {
					$new_item['terms'] = $tax_query_item['terms'];
				}
				// TODO.
				if ( isset( $tax_query_item['includeChildren'] ) ) {
					$new_item['include_children'] = $tax_query_item['includeChildren'];
				}
				$query['tax_query'][] = $new_item;
			}
		}
		return $query;
	}

	/*
	// TODO - check if we should uncomment this? Seems like we should.
	public function get_sort_order_args( $data ) {
		$order = explode( '-', $data['sortOrder'] );
		if ( $order[0] === 'inherit' ) {
			return array();
		}
		$query_args = array(
			'orderby' => $order[0],
			'order'   => $order[1],
		);
		return $query_args;
	}
	*/
	/**
	 * Sets the Query context (single, archive etc)
	 *
	 * @param string $context The context to set
	 */
	public function set_context( $context ) {
		$this->context = $context;
	}
	/**
	 * Gets the Query context (theme)
	 */
	public function get_context() {
		return $this->context;
	}
	/**
	 * Get the css for the query
	 */
	public function get_css() {
		return $this->css;
	}
	/**
	 * Set the css for the query
	 *
	 * @param string $css The css to set.
	 */
	public function set_css( $css ) {
		$this->css = $css;
	}
	public function get_fields() {
		$fields = Fields::find(
			array(
				'query_id' => $this->get_id(),
				'status'   => 'enabled',
			)
		);
		return $fields;
	}


	/**
	 * Sets the Query context path - eg 'post/123'
	 *
	 * @param string $context The context to set.
	 * @param string $path The path of the context.
	 */
	public function set_integration( $path ) {
		$this->integration = $path;
	}


	/**
	 * Gets the Query integration path
	 */
	public function get_integration() {
		return $this->integration;
	}
	/**
	 * Sets the Query context from attributes
	 *
	 * @param string $context The context to set.
	 * @param string $path The path of the context.
	 */
	public function generate_context() {
		$integration_type = $this->attributes['integrationType'];

		$integration = $integration_type;
		if ( $integration_type === 'archive' ) {
			$integration .= "/{$this->attributes['postType']}";
		} elseif ( $integration_type === 'single' ) {
			$integration .= "/{$this->attributes['singleLocation']}";
		}
		$this->set_integration( $integration );
	}

	/**
	 * Saves the query
	 *
	 * @return int The saved query ID.
	 */
	public function save( $args = array() ) {

		if ( $this->get_id() === 0 ) {
			// If this is a new record, save it to get an ID.
			$id = parent::save();
		}
		// Update the CSS using the ID.
		$this->regenerate_css();

		// Prepare the full record.
		$extra_args = array(
			'context'     => $this->get_context(),
			'integration' => $this->get_integration(),
			'css'         => $this->get_css(),
		);

		// TODO - are we clearing our Data_Store once the info has been updated?
		// Or shall we update it rather than clear it?
		$id = parent::save( $extra_args );

		// We want to store the integration data as meta so we can query it.
		// Eg, get all queries that are connected to the query loop block, etc.
		self::update_meta( $id, 'integration_type', $this->get_attribute( 'integrationType' ) );
		self::update_meta( $id, 'single_location', $this->get_attribute( 'singleLocation' ) );
		self::update_meta( $id, 'single_integration', $this->get_attribute( 'singleIntegration' ) );
		self::update_meta( $id, 'query_loop_id', $this->get_attribute( 'queryLoopId' ) );
		self::update_meta( $id, 'archive_type', $this->get_attribute( 'archiveType' ) );
		self::update_meta( $id, 'archive_post_type', $this->get_attribute( 'postType' ) );
		
		return $id;
	}

	/**
	 * Regenerates the CSS for the style preset.
	 *
	 * @since   3.0.0
	 */
	public function regenerate_css() {
		$this->css = $this->generate_css();
	}

	/**
	 * Generates the CSS for the query.
	 *
	 * @return string The generated CSS.
	 */
	private function generate_css() {
		$css = '';
		// Get the base styles class for the ID.
		$query_class = '.search-filter--query-' . intval( $this->get_id() );

		$css .= $query_class . '{';
		$css .= CSS_Loader::clean_css( $this->create_attributes_css( $this->get_attributes() ) );
		$css .= '}';

		return $css;
	}
	/**
	 * Parses the attributes into CSS styles (variables).
	 *
	 * @since   3.0.0
	 *
	 * @param array $attributes  The saved style attributes.
	 *
	 * @return string The generated CSS.
	 */
	public function create_attributes_css( $attributes ) {
		$css = apply_filters( 'search-filter/queries/query/create_attributes_css', '', $attributes );
		return $css;
	}

	public function get_attributes() {
		$attributes = parent::get_attributes();
		$attributes = apply_filters( "search-filter/queries/query/get_attributes", $attributes, $this->get_id() );
		return $attributes;
	}
}
